@@ -25,10 +25,10 @@ use git_wrapper::{
 };
 use std::path::{Path, PathBuf};
 
-use posix_errors::{PosixError, ENOENT};
+use posix_errors::{PosixError, EAGAIN, EINVAL, ENOENT, ENOTRECOVERABLE, ENOTSUP, EUTF8};
 
 /// Configuration for a subtree
-#[derive(Getters, Clone, Eq, PartialEq)]
+#[derive(Getters, Clone, Debug, Eq, PartialEq)]
 pub struct SubtreeConfig {
     /// subtree id
     #[getset(get = "pub")]
@@ -48,8 +48,10 @@ pub struct SubtreeConfig {
 }
 
 impl SubtreeConfig {
+    /// Return a new instance
     #[must_use]
-    pub fn new(
+    #[inline]
+    pub const fn new(
         id: String,
         follow: Option<String>,
         origin: Option<String>,
@@ -66,31 +68,37 @@ impl SubtreeConfig {
     }
     /// Return `true` if upstream is set
     #[must_use]
-    pub fn is_pullable(&self) -> bool {
+    #[inline]
+    pub const fn is_pullable(&self) -> bool {
         self.upstream.is_some()
     }
 
     /// Return `true` if origin is set
     #[must_use]
-    pub fn is_pushable(&self) -> bool {
+    #[inline]
+    pub const fn is_pushable(&self) -> bool {
         self.origin.is_some()
     }
 
+    /// Return path to config file
     #[must_use]
+    #[inline]
     pub fn config_file(&self) -> String {
         let mut result = self
             .id
             .rsplit_once('/')
-            .map_or_else(|| "".to_string(), |x| x.1.to_string());
+            .map_or_else(|| "".to_owned(), |x| x.1.to_owned());
         result.push_str(".gitsubtrees");
         result
     }
 
+    /// Subtree name
     #[must_use]
+    #[inline]
     pub fn name(&self) -> String {
         self.id()
             .rsplit_once('/')
-            .map_or_else(|| self.id.to_string(), |x| x.1.to_string())
+            .map_or_else(|| self.id.clone(), |x| x.1.to_owned())
     }
 
     /// Figure out which named ref to pull from.
@@ -102,11 +110,12 @@ impl SubtreeConfig {
     /// # Errors
     ///
     /// Will return a [`PosixError`] when fails to find a ref to pull
+    #[inline]
     pub fn ref_to_pull(&self) -> Result<String, PosixError> {
         if !self.is_pullable() {
             panic!("Subtree does not have upstream remote defined");
         }
-        let candidate = self.follow.clone().unwrap_or_else(|| "HEAD".to_string());
+        let candidate = self.follow.clone().unwrap_or_else(|| "HEAD".to_owned());
         let remote = &self.upstream.clone().unwrap();
         let follow = if candidate == *"@{tags}" {
             find_latest_version(remote)?
@@ -129,6 +138,7 @@ impl SubtreeConfig {
 
 /// Aliases some well known urls to their initials.
 #[must_use]
+#[inline]
 pub fn alias_url(url: &str) -> String {
     let github = regex::Regex::new(r"^(git@github.com:|.+://github.com/)").expect("Valid RegEx");
     let gitlab = regex::Regex::new(r"^(git@gitlab.com:|.+://gitlab.com/)").expect("Valid RegEx");
@@ -143,7 +153,7 @@ pub fn alias_url(url: &str) -> String {
     if bitbucket.is_match(url) {
         return bitbucket.replace(url, "BB:").to_string();
     }
-    url.to_string()
+    url.to_owned()
 }
 
 fn versions_from_remote(url: &str) -> Result<HashMap<semver::Version, String>, PosixError> {
@@ -165,10 +175,11 @@ fn versions_from_remote(url: &str) -> Result<HashMap<semver::Version, String>, P
 /// # Errors
 ///
 /// Will return [`PosixError`] if command exits if no versions found.
+#[inline]
 pub fn find_latest_version(remote: &str) -> Result<String, PosixError> {
     let versions = versions_from_remote(remote)?;
     if versions.is_empty() {
-        let message = "Failed to find any valid tags".to_string();
+        let message = "Failed to find any valid tags".to_owned();
         return Err(PosixError::new(ENOENT, message));
     }
 
@@ -186,6 +197,7 @@ pub fn find_latest_version(remote: &str) -> Result<String, PosixError> {
 /// # Errors
 ///
 /// Will return [`PosixError`] if command exits if fails to find matching version.
+#[inline]
 pub fn find_latest_version_matching(
     remote: &str,
     range: &semver::VersionReq,
@@ -212,22 +224,27 @@ pub fn find_latest_version_matching(
         } else {
         }
     }
-    if let Some(v) = latest {
-        let result = versions_map.get(v);
-        Ok(result.expect("Version is in version map").clone())
-    } else {
-        let msg = format!("Failed to find a tag matching {}", range);
-        Err(PosixError::new(ENOENT, msg))
-    }
+    latest.map_or_else(
+        || {
+            let msg = format!("Failed to find a tag matching {}", range);
+            Err(PosixError::new(ENOENT, msg))
+        },
+        |v| {
+            let result = versions_map.get(v);
+            Ok(result.expect("Version is in version map").clone())
+        },
+    )
 }
 
 /// Manages subtrees in a repository
+#[derive(Debug)]
 pub struct Subtrees {
     repo: Repository,
     configs: Vec<SubtreeConfig>,
 }
 
 /// Failed to initialize `Subtrees`
+#[allow(missing_docs)]
 #[derive(Debug, PartialEq)]
 pub enum SubtreesError {
     RepoNotFound,
@@ -236,31 +253,32 @@ pub enum SubtreesError {
 }
 
 impl From<RepoError> for SubtreesError {
+    #[inline]
     fn from(err: RepoError) -> Self {
         match err {
-            RepoError::GitDirNotFound => SubtreesError::RepoNotFound,
+            RepoError::GitDirNotFound => Self::RepoNotFound,
             RepoError::InvalidDirectory(p) | RepoError::AbsolutionError(p) => {
                 let msg = format!("Failed to handle directory {:?}", p);
-                SubtreesError::InvalidDirectory(msg)
-            }
-            RepoError::FailAccessCwd => {
-                SubtreesError::InvalidDirectory("Failed to access CWD".to_string())
+                Self::InvalidDirectory(msg)
             }
+            RepoError::FailAccessCwd => Self::InvalidDirectory("Failed to access CWD".to_owned()),
         }
     }
 }
 
 impl From<ConfigError> for SubtreesError {
+    #[inline]
     fn from(err: ConfigError) -> Self {
         match err {
             ConfigError::ParseFailed(path_buf) | ConfigError::ReadFailed(path_buf) => {
-                SubtreesError::InvalidConfig(path_buf.to_string_lossy().to_string())
+                Self::InvalidConfig(path_buf.to_string_lossy().to_string())
             }
         }
     }
 }
 
 /// Failed reading or parsing a `.gitsubtrees` file.
+#[allow(missing_docs)]
 #[derive(Debug, PartialEq)]
 pub enum ConfigError {
     ReadFailed(PathBuf),
@@ -268,6 +286,7 @@ pub enum ConfigError {
 }
 
 /// Failed adding a new subtree to a repository fails
+#[allow(missing_docs)]
 #[derive(Debug, PartialEq)]
 pub enum AdditionError {
     BareRepository,
@@ -277,33 +296,57 @@ pub enum AdditionError {
     Failure(String, i32),
 }
 
+impl From<AdditionError> for PosixError {
+    #[inline]
+    fn from(err: AdditionError) -> Self {
+        match err {
+            AdditionError::BareRepository => {
+                let msg = "Operation on bare repository not supported".to_owned();
+                Self::new(ENOTSUP, msg)
+            }
+            AdditionError::WorkTreeDirty => {
+                let msg = "Working tree is dirty".to_owned();
+                Self::new(ENOTSUP, msg)
+            }
+            AdditionError::InvalidVersion(version) => {
+                let msg = format!("Invalid version {}", version);
+                Self::new(EINVAL, msg)
+            }
+            AdditionError::Failure(msg, _) | AdditionError::WriteConfig(msg) => Self::new(1, msg),
+        }
+    }
+}
+
 impl From<SubtreeAddError> for AdditionError {
+    #[inline]
     fn from(err: SubtreeAddError) -> Self {
         match err {
-            SubtreeAddError::BareRepository => AdditionError::BareRepository,
-            SubtreeAddError::Failure(msg, code) => AdditionError::Failure(msg, code),
-            SubtreeAddError::WorkTreeDirty => AdditionError::WorkTreeDirty,
+            SubtreeAddError::BareRepository => Self::BareRepository,
+            SubtreeAddError::Failure(msg, code) => Self::Failure(msg, code),
+            SubtreeAddError::WorkTreeDirty => Self::WorkTreeDirty,
         }
     }
 }
 
 impl From<ConfigSetError> for AdditionError {
+    #[inline]
     fn from(err: ConfigSetError) -> Self {
         match err {
             ConfigSetError::InvalidConfigFile(f) => {
                 let msg = format!("Invalid config file: {}", f);
-                AdditionError::WriteConfig(msg)
+                Self::WriteConfig(msg)
             }
             ConfigSetError::WriteFailed(f) => {
                 let msg = format!("Failed to write config file: {}", f);
-                AdditionError::WriteConfig(msg)
+                Self::WriteConfig(msg)
             }
-            ConfigSetError::InvalidSectionOrKey(msg) => AdditionError::WriteConfig(msg),
+            ConfigSetError::InvalidSectionOrKey(msg) => Self::WriteConfig(msg),
         }
     }
 }
 
 /// Failed to find specified subtree
+#[allow(missing_docs)]
 #[derive(Debug, PartialEq)]
 pub enum FindError {
     BareRepository,
@@ -313,57 +356,97 @@ pub enum FindError {
 }
 
 impl From<ConfigError> for FindError {
+    #[inline]
     fn from(err: ConfigError) -> Self {
         match err {
-            ConfigError::ReadFailed(p) => FindError::ReadFailed(p),
-            ConfigError::ParseFailed(p) => FindError::ParseFailed(p),
+            ConfigError::ReadFailed(p) => Self::ReadFailed(p),
+            ConfigError::ParseFailed(p) => Self::ParseFailed(p),
         }
     }
 }
 
 /// Failed to update a subtree from remote
 #[derive(Debug)]
+#[allow(missing_docs)]
 pub enum PullError {
     BareRepository,
     Failure(String),
     IOError(std::io::Error),
     InvalidReference,
+    NoChanges,
     NoUpstream,
     ReferenceNotFound,
     UTF8Decode(std::string::FromUtf8Error),
     WorkTreeDirty,
 }
 
+impl From<PullError> for PosixError {
+    #[inline]
+    fn from(e: PullError) -> Self {
+        match e {
+            PullError::BareRepository => {
+                let msg = "Can not execute pull operation in bare repository".to_owned();
+                Self::new(ENOENT, msg)
+            }
+            PullError::WorkTreeDirty => {
+                let msg = "Can not execute pull operation in a dirty repository".to_owned();
+                Self::new(ENOENT, msg)
+            }
+            PullError::ReferenceNotFound | PullError::InvalidReference => {
+                Self::new(EINVAL, "Can not find reference".to_owned())
+            }
+            PullError::UTF8Decode(_) => {
+                let msg = "Failed UTF8 decoding".to_owned();
+                Self::new(EUTF8, msg)
+            }
+            PullError::NoChanges => {
+                let msg = "Upstream does not have any new changes".to_owned();
+                Self::new(EAGAIN, msg)
+            }
+            PullError::NoUpstream => {
+                let msg = "Subtree does not have a upstream defined".to_owned();
+                Self::new(ENOTRECOVERABLE, msg)
+            }
+            PullError::Failure(msg) => Self::new(1, msg),
+            PullError::IOError(err) => Self::from(err),
+        }
+    }
+}
+
 impl From<RefSearchError> for PullError {
+    #[inline]
     fn from(prev: RefSearchError) -> Self {
         match prev {
             RefSearchError::ParsingFailure(msg) | RefSearchError::Failure(msg) => {
-                PullError::Failure(msg)
+                Self::Failure(msg)
             }
-            RefSearchError::UTF8Decode(err) => PullError::UTF8Decode(err),
-            RefSearchError::IOError(err) => PullError::IOError(err),
-            RefSearchError::NotFound => PullError::ReferenceNotFound,
+            RefSearchError::UTF8Decode(err) => Self::UTF8Decode(err),
+            RefSearchError::IOError(err) => Self::IOError(err),
+            RefSearchError::NotFound => Self::ReferenceNotFound,
         }
     }
 }
 
 impl From<InvalidRefError> for PullError {
+    #[inline]
     fn from(_prev: InvalidRefError) -> Self {
-        PullError::InvalidReference
+        Self::InvalidReference
     }
 }
 
 impl From<SubtreePullError> for PullError {
+    #[inline]
     fn from(prev: SubtreePullError) -> Self {
         match prev {
-            SubtreePullError::Failure(msg, _) => PullError::Failure(msg),
-            SubtreePullError::BareRepository => PullError::BareRepository,
-            SubtreePullError::WorkTreeDirty => PullError::WorkTreeDirty,
+            SubtreePullError::Failure(msg, _) => Self::Failure(msg),
+            SubtreePullError::BareRepository => Self::BareRepository,
+            SubtreePullError::WorkTreeDirty => Self::WorkTreeDirty,
         }
     }
 }
 
 /// Failed to push subtree to remote
+#[allow(missing_docs)]
 #[derive(Debug)]
 pub enum PushError {
     BareRepository,
@@ -371,16 +454,37 @@ pub enum PushError {
     Failure(String),
 }
 
+impl From<PushError> for PosixError {
+    #[inline]
+    fn from(err: PushError) -> Self {
+        match err {
+            PushError::BareRepository => {
+                let msg = "Can not execute push operation in bare repository".to_owned();
+                Self::new(ENOENT, msg)
+            }
+
+            PushError::NoUpstream => {
+                let msg = "Subtree does not have a upstream defined".to_owned();
+                Self::new(ENOTRECOVERABLE, msg)
+            }
+
+            PushError::Failure(msg) => Self::new(1, msg),
+        }
+    }
+}
+
 impl From<SubtreePushError> for PushError {
+    #[inline]
     fn from(prev: SubtreePushError) -> Self {
         match prev {
-            SubtreePushError::Failure(msg, _) => PushError::Failure(msg),
-            SubtreePushError::BareRepository => PushError::BareRepository,
+            SubtreePushError::Failure(msg, _) => Self::Failure(msg),
+            SubtreePushError::BareRepository => Self::BareRepository,
         }
     }
 }
 
 /// Failed to split subtree
+#[allow(missing_docs)]
 #[derive(Debug)]
 pub enum SplitError {
     BareRepository,
@@ -388,18 +492,37 @@ pub enum SplitError {
     Failure(String),
 }
 
+impl From<SplitError> for PosixError {
+    #[inline]
+    fn from(err: SplitError) -> Self {
+        match err {
+            SplitError::BareRepository => {
+                let msg = "Can not execute push operation in bare repository".to_owned();
+                Self::new(ENOENT, msg)
+            }
+            SplitError::WorkTreeDirty => {
+                let msg = "Can not execute push operation in a dirty repository".to_owned();
+                Self::new(ENOENT, msg)
+            }
+            SplitError::Failure(msg) => Self::new(1, msg),
+        }
+    }
+}
+
 impl From<SubtreeSplitError> for SplitError {
+    #[inline]
     fn from(prev: SubtreeSplitError) -> Self {
         match prev {
-            SubtreeSplitError::Failure(msg, _) => SplitError::Failure(msg),
-            SubtreeSplitError::BareRepository => SplitError::BareRepository,
-            SubtreeSplitError::WorkTreeDirty => SplitError::WorkTreeDirty,
+            SubtreeSplitError::Failure(msg, _) => Self::Failure(msg),
+            SubtreeSplitError::BareRepository => Self::BareRepository,
+            SubtreeSplitError::WorkTreeDirty => Self::WorkTreeDirty,
         }
     }
 }
 
 #[cfg(not(tarpaulin_include))]
 impl std::fmt::Display for ConfigError {
+    #[inline]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             ConfigError::ParseFailed(p) => {
@@ -419,17 +542,22 @@ impl Subtrees {
     /// # Errors
     ///
     /// Throws [`SubtreesError`] if fails to find or access repository.
+    #[inline]
     pub fn new() -> Result<Self, SubtreesError> {
         let repo = Repository::default()?;
         let configs = all(&repo)?;
         Ok(Self { repo, configs })
     }
 
+    /// Discover subtrees in a git repository
+    #[inline]
     pub fn from_repo(repo: Repository) -> Result<Self, SubtreesError> {
         let configs = all(&repo)?;
         Ok(Self { repo, configs })
     }
 
+    /// Discover subtrees in specified directory
+    #[inline]
     pub fn from_dir(path: &Path) -> Result<Self, SubtreesError> {
         let repo = Repository::discover(path)?;
         let configs = all(&repo)?;
@@ -443,17 +571,18 @@ impl Subtrees {
     /// # Panics
     ///
     /// Panics when something unexpected happens
+    #[inline]
     pub fn add(
         &self,
         subtree: &SubtreeConfig,
         revision: Option<&str>,
         subject: Option<&str>,
     ) -> Result<(), AdditionError> {
-        if let Some(revision) = revision {
+        if let Some(rev) = revision {
             let remote = subtree.upstream.as_ref().unwrap();
             let target = subtree.id();
 
-            let subject = subject.map_or_else(
+            let title = subject.map_or_else(
                 || format!(":{} Import {}", target, alias_url(remote)),
                 |v| format!(":{} {}", target, v),
             );
@@ -462,9 +591,9 @@ impl Subtrees {
 
 git-subtree-origin: {}
 git-subtree-remote-ref: {}",
-                subject, remote, revision
+                title, remote, rev
             );
-            self.repo.subtree_add(remote, target, revision, &msg)?;
+            self.repo.subtree_add(remote, target, rev, &msg)?;
         }
         self.persist(subtree)?;
         let mut cmd = self.repo.git();
@@ -475,10 +604,10 @@ git-subtree-remote-ref: {}",
             return Err(AdditionError::WriteConfig(msg));
         }
 
-        let mut cmd = self.repo.git();
-        cmd.args(&["commit", "--amend", "--no-edit"]);
-        let out = cmd.output().expect("Failed to execute git-commit(1)");
-        if !out.status.success() {
+        let mut cmd2 = self.repo.git();
+        cmd2.args(&["commit", "--amend", "--no-edit"]);
+        let out2 = cmd2.output().expect("Failed to execute git-commit(1)");
+        if !out2.status.success() {
             let msg = String::from_utf8_lossy(&out.stderr).to_string();
             return Err(AdditionError::WriteConfig(msg));
         }
@@ -488,16 +617,19 @@ git-subtree-remote-ref: {}",
     /// # Errors
     ///
     /// Throws [`ConfigError`] if something goes wrong during parsing
+    #[inline]
     pub fn all(&self) -> Result<Vec<SubtreeConfig>, ConfigError> {
         Ok(self.configs.clone())
     }
 
+    /// Returns the repository head commit id
     #[must_use]
+    #[inline]
     pub fn head(&self) -> Option<String> {
         self.repo.head()
     }
 
-    fn persist(&self, subtree: &SubtreeConfig) -> Result<(), git_wrapper::ConfigSetError> {
+    fn persist(&self, subtree: &SubtreeConfig) -> Result<(), ConfigSetError> {
         let root = self.repo.work_tree().expect("Repo without work_tree");
         let file = root.join(subtree.config_file());
         let section = subtree.name();
@@ -534,6 +666,8 @@ git-subtree-remote-ref: {}",
         Ok(())
     }
 
+    /// Pull remote changes in the specified subtree
+    #[inline]
     pub fn pull(&self, subtree: &SubtreeConfig, git_ref: &str) -> Result<String, PullError> {
         let prefix = subtree.id();
         let remote;
@@ -544,7 +678,12 @@ git-subtree-remote-ref: {}",
         }
 
         let message = format!("Update :{} to {}", prefix, &git_ref);
+        let head_before = self.repo.head().expect("HEAD ref exists");
         self.repo.subtree_pull(remote, prefix, git_ref, &message)?;
+        let head_after = self.repo.head().expect("HEAD ref exists");
+        if head_before == head_after {
+            return Err(PullError::NoChanges);
+        }
         let mut cmd = self.repo.git();
         let out = cmd
             .arg("rev-parse")
@@ -556,18 +695,23 @@ git-subtree-remote-ref: {}",
             Ok(String::from_utf8(out.stdout)
                 .expect("UTF-8 encoding")
                 .trim()
-                .to_string())
+                .to_owned())
         } else {
-            eprintln!("Failed to execute git rev-parse");
-            Ok(git_ref.to_string())
+            Err(PullError::Failure(
+                "Failed to execute git rev-parse".to_owned(),
+            ))
         }
     }
 
+    /// Split changes in a subtree to own artificial history and merge it back into HEAD
+    #[inline]
     pub fn split(&self, subtree: &SubtreeConfig) -> Result<(), SplitError> {
         let prefix = subtree.id();
         Ok(self.repo.subtree_split(prefix)?)
     }
 
+    /// Push subtree changes to remote
+    #[inline]
     pub fn push(&self, subtree: &SubtreeConfig, git_ref: &str) -> Result<(), PushError> {
         let prefix = subtree.id();
         let remote;
@@ -577,9 +721,16 @@ git-subtree-remote-ref: {}",
             return Err(PushError::NoUpstream);
         }
 
-        Ok(self.repo.subtree_push(remote, prefix, git_ref)?)
+        if git_ref == "HEAD" {
+            let head = git_wrapper::resolve_head(remote).expect("asd");
+            Ok(self.repo.subtree_push(remote, prefix, &head)?)
+        } else {
+            Ok(self.repo.subtree_push(remote, prefix, git_ref)?)
+        }
     }
 
+    /// List modules with changes since specified git commit id
+    #[inline]
     pub fn changed_modules(&self, id: &str) -> Result<Vec<SubtreeConfig>, ConfigError> {
         let subtree_modules = self.all()?;
         if subtree_modules.is_empty() {
@@ -623,7 +774,9 @@ git-subtree-remote-ref: {}",
         Ok(result)
     }
 
+    /// Find subtree by name
     #[allow(clippy::missing_panics_doc)]
+    #[inline]
     pub fn find_subtree(&self, needle: &str) -> Result<SubtreeConfig, FindError> {
         if self.repo.is_bare() {
             return Err(FindError::BareRepository);
@@ -635,14 +788,14 @@ git-subtree-remote-ref: {}",
                 return Ok(c);
             }
         }
-        Err(FindError::NotFound(needle.to_string()))
+        Err(FindError::NotFound(needle.to_owned()))
     }
 }
 
 fn configs_from_path(
     repo: &Repository,
     parser: &mut Ini,
-    path: &std::path::Path,
+    path: &Path,
 ) -> Result<Vec<SubtreeConfig>, ConfigError> {
     let content;
     match repo.hack_read_file(path) {
@@ -662,7 +815,7 @@ fn configs_from_path(
                 .join(name)
                 .to_str()
                 .expect("Convertable to str")
-                .to_string();
+                .to_owned();
         } else {
             id = name.clone();
         }
@@ -713,30 +866,12 @@ fn all(repo: &Repository) -> Result<Vec<SubtreeConfig>, ConfigError> {
     Ok(result)
 }
 
-#[cfg(not(tarpaulin_include))]
 #[cfg(test)]
 mod test {
     use crate::SubtreeConfig;
     use crate::Subtrees;
     use git_wrapper::Repository;
-    use tempdir::TempDir;
-
-    // https://stackoverflow.com/a/63904992
-    macro_rules! function {
-        () => {{
-            fn f() {}
-            fn type_name_of<T>(_: T) -> &'static str {
-                std::any::type_name::<T>()
-            }
-            let name = type_name_of(f);
-
-            // Find and cut the rest of the path
-            match &name[..name.len() - 3].rfind(':') {
-                Some(pos) => &name[pos + 1..name.len() - 3],
-                None => &name[..name.len() - 3],
-            }
-        }};
-    }
+    use tempfile::TempDir;
 
     #[test]
     fn bkg_monorepo() {
@@ -766,7 +901,7 @@ mod test {
 
     #[test]
     fn initialization() {
-        let tmp_dir = TempDir::new(function!()).unwrap();
+        let tmp_dir = TempDir::new().unwrap();
         let repo_path = tmp_dir.path();
         let _ = Repository::create_bare(repo_path).unwrap();
         let actual = Subtrees::from_dir(&repo_path);
@@ -775,7 +910,7 @@ mod test {
 
     #[test]
     fn subtree_add() {
-        let tmp_dir = TempDir::new(function!()).unwrap();
+        let tmp_dir = TempDir::new().unwrap();
         let repo_path = tmp_dir.path();
         {
             let repo = Repository::create(repo_path).unwrap();
@@ -799,7 +934,7 @@ mod test {
 
     #[test]
     fn subtree_pull() {
-        let tmp_dir = TempDir::new(function!()).unwrap();
+        let tmp_dir = TempDir::new().unwrap();
         let repo_path = tmp_dir.path();
         {
             let repo = Repository::create(repo_path).unwrap();
@@ -817,8 +952,12 @@ mod test {
             upstream: Some("https://github.com/kalkin/file-expert".to_string()),
             pull_pre_releases: false,
         };
-        mgr.add(&config, Some("master"), None).unwrap();
+        mgr.add(&config, Some("v0.10.1"), None).unwrap();
         let actual = mgr.pull(&config, "v0.13.1");
-        assert!(actual.is_ok(), "Expected successful pull execution");
+        assert!(
+            actual.is_ok(),
+            "Expected successful pull execution, got: {:?}",
+            actual
+        );
     }
 }
